###
# #%L
# aiSSEMBLE::Test::MDA::Data Delivery Pyspark
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from abc import ABC

from pyspark.sql.dataframe import DataFrame
from pyspark.sql.column import Column
from pyspark.sql.types import StructType
from pyspark.sql.types import DataType
from pyspark.sql.functions import col, lit
from typing import List
import types
from pyspark.sql.types import StringType




class AddressSchemaBase(ABC):
    """
    Base implementation of the PySpark schema for Address.

    GENERATED CODE - DO NOT MODIFY (add your customizations in Address).

    Generated from: templates/data-delivery-data-records/pyspark.schema.base.py.vm
    """

    STREET_COLUMN: str = 'street'
    CITY_COLUMN: str = 'city'
    ZIPCODE_COLUMN: str = 'zipcode'
    STATE_COLUMN: str = 'state'


    def __init__(self):
        self._schema = StructType()

        self.add(AddressSchemaBase.STREET_COLUMN, StringType(), True)
        self.add(AddressSchemaBase.CITY_COLUMN, StringType(), True)
        self.add(AddressSchemaBase.ZIPCODE_COLUMN, StringType(), True)
        self.add(AddressSchemaBase.STATE_COLUMN, StringType(), True)

    def cast(self, dataset: DataFrame) -> DataFrame:
        """
        Returns the given dataset cast to this schema.
        """
        street_type = self.get_data_type(AddressSchemaBase.STREET_COLUMN)
        city_type = self.get_data_type(AddressSchemaBase.CITY_COLUMN)
        zipcode_type = self.get_data_type(AddressSchemaBase.ZIPCODE_COLUMN)
        state_type = self.get_data_type(AddressSchemaBase.STATE_COLUMN)

        return dataset \
            .withColumn(AddressSchemaBase.STREET_COLUMN, dataset[AddressSchemaBase.STREET_COLUMN].cast(street_type)) \
            .withColumn(AddressSchemaBase.CITY_COLUMN, dataset[AddressSchemaBase.CITY_COLUMN].cast(city_type)) \
            .withColumn(AddressSchemaBase.ZIPCODE_COLUMN, dataset[AddressSchemaBase.ZIPCODE_COLUMN].cast(zipcode_type)) \
            .withColumn(AddressSchemaBase.STATE_COLUMN, dataset[AddressSchemaBase.STATE_COLUMN].cast(state_type))


    @property
    def struct_type(self) -> StructType:
        """
        Returns the structure type for this schema.
        """
        return self._schema


    @struct_type.setter
    def struct_type(self, struct_type: StructType) -> None:
        raise Exception('Schema structure type should not be set manually!')


    def get_data_type(self, name: str) -> str:
        """
        Returns the data type for a field in this schema.
        """
        data_type = None
        if name in self._schema.fieldNames():
            data_type = self._schema[name].dataType

        return data_type


    def add(self, name: str, data_type: DataType, nullable: bool) -> None:
        """
        Adds a field to this schema.
        """
        self._schema.add(name, data_type, nullable)


    def update(self, name: str, data_type: DataType) -> None:
        """
        Updates the data type of a field in this schema.
        """
        fields = self._schema.fields
        if fields and len(fields) > 0:
            update = StructType()
            for field in fields:
                if field.name == name:
                    update.add(name, data_type, field.nullable)
                else:
                    update.add(field)

            self._schema = update

    def validate_dataset(self, ingest_dataset: DataFrame) -> DataFrame:
        return self.validate_dataset_with_prefix(ingest_dataset, "")

    def validate_dataset_with_prefix(self, ingest_dataset: DataFrame, column_prefix: str) -> DataFrame:
        """
        Validates the given dataset and returns the lists of validated records.
        """
        data_with_validations = ingest_dataset
        data_with_validations = data_with_validations.withColumn("ZIPCODE_GREATER_THAN_OR_EQUAL_TO_MIN_LENGTH", col(column_prefix + "ZIPCODE").rlike("^.{5,}"))
        data_with_validations = data_with_validations.withColumn("ZIPCODE_LESS_THAN_OR_EQUAL_TO_MAX_LENGTH", col(column_prefix + "ZIPCODE").rlike("^.{11,}").eqNullSafe(False))
        data_with_validations = data_with_validations.withColumn("ZIPCODE_MATCHES_FORMAT", col(column_prefix + "ZIPCODE").rlike("^\\d{5}(?:[-\\s]\\d{4})?$"))
        data_with_validations = data_with_validations.withColumn("STATE_GREATER_THAN_OR_EQUAL_TO_MIN_LENGTH", col(column_prefix + "STATE").rlike("^.{2,}"))
        data_with_validations = data_with_validations.withColumn("STATE_LESS_THAN_OR_EQUAL_TO_MAX_LENGTH", col(column_prefix + "STATE").rlike("^.{3,}").eqNullSafe(False))

    
        validation_columns = [x for x in data_with_validations.columns if x not in ingest_dataset.columns]

        # Schema for filtering for valid data
        filter_schema = None
        for column_name in validation_columns:
            if isinstance(filter_schema, Column):
                filter_schema = filter_schema & col(column_name).eqNullSafe(True)
            else:
                filter_schema = col(column_name).eqNullSafe(True)

        valid_data = data_with_validations
        # Isolate the valid data and drop validation columns
        if isinstance(filter_schema, Column):
            valid_data = data_with_validations.filter(filter_schema)
        valid_data = valid_data.drop(*validation_columns)
        return valid_data



